@@ -76,6 +76,8 @@ namespace eval reclaim {
         uninstall_unrequested
         uninstall_inactive
         remove_distfiles
+        remove_builds
+        remove_ccache
 
         if {![macports::global_option_isset ports_dryrun]} {
             set last_run_contents [read_last_run_file]
@@ -104,6 +106,82 @@ namespace eval reclaim {
         }
     }
 
+    proc remove_builds {} {
+        # Delete portdbpath/build directories
+        global macports::portdbpath
+
+        # The root build directory location
+        set root_build      [file join ${macports::portdbpath} build]
+
+        ui_msg "$macports::ui_prefix Build location: ${root_build}"
+
+        if {[macports::global_option_isset ports_dryrun]} {
+            ui_msg "Deleting... (dry run)"
+            ui_info [msgcat::mc "Skipping deletion of all build directories under %s (dry run)" $root_build]
+            return
+        }
+
+        set builddirs [glob -nocomplain -directory $root_build *]
+        if {[llength $builddirs] == 0} {
+            ui_info [msgcat::mc "No build directories to delete"]
+            return
+        }
+
+        set retval 0
+        if {[info exists macports::ui_options(questions_yesno)]} {
+            set retval [$macports::ui_options(questions_yesno) "" "" "" "y" 0 "Would you like to delete all the build directories?"]
+        }
+
+        if {${retval} == 0} {
+            ui_info [msgcat::mc "Deleting all build directories under %s" $root_build]
+            try -pass_signal {
+                file delete -force -- {*}$builddirs
+            } catch {{*} eCode eMessage} {
+                ui_debug "$::errorInfo"
+                ui_error "$eMessage"
+            }
+        }
+    }
+
+    proc remove_ccache {} {
+        # Delete everything under ccache directory - default is build/.ccache
+        global macports::ccache_dir
+
+        if {![file exists ${macports::ccache_dir}]} {
+            ui_info [msgcat::mc "Skipping deletion of ccache directory: %s does not exist." $macports::ccache_dir]
+            return
+        }
+
+        ui_msg "$macports::ui_prefix ccache location: ${macports::ccache_dir}"
+
+        if {[macports::global_option_isset ports_dryrun]} {
+            ui_msg "Deleting... (dry run)"
+            ui_info [msgcat::mc "Skipping deletion of everything under %s (dry run)" $macports::ccache_dir]
+            return
+        }
+
+        set ccachedirs [glob -nocomplain -directory $macports::ccache_dir *]
+        if {[llength $ccachedirs] == 0} {
+            ui_info [msgcat::mc "No ccache directories to delete"]
+            return
+        }
+
+        set retval 0
+        if {[info exists macports::ui_options(questions_yesno)]} {
+            set retval [$macports::ui_options(questions_yesno) "" "" "" "y" 0 "Would you like to delete everything under the ccache directory?"]
+        }
+
+        if {${retval} == 0} {
+            ui_info [msgcat::mc "Deleting everything under %s" $macports::ccache_dir]
+            try -pass_signal {
+                file delete -force -- {*}$ccachedirs
+            } catch {{*} eCode eMessage} {
+                ui_debug "$::errorInfo"
+                ui_error "$eMessage"
+            }
+        }
+    }
+
     proc walk_files {dir files_in_use unused_name} {
         # Recursively walk the given directory $dir and build a list of all files that are present on-disk but not listed in $files_in_use.
         # The list of unused files will be stored in the variable given by $unused_name
@@ -122,10 +200,13 @@ namespace eval reclaim {
                     walk_files $currentPath $files_in_use unused
                 }
                 file {
-                    if {$item eq ".turd_MacPorts"} {
+                    if {$item eq ".turd_MacPorts" || $item eq ".DS_Store"} {
                         # .turd_MacPorts files are created by MacPorts when creating the MacPorts
                         # installer packages from the MacPorts port so that empty directories are
-                        # not deleted after destroot. Treat those files as if they were not there.
+                        # not deleted after destroot.
+                        # .DS_Store files are created by the OS that stores custom attributes of
+                        # its containing folder,
+                        # Treat those files as if they were not there.
                         continue
                     }
                     if {[lsearch -exact -sorted $files_in_use $currentPath] == -1} {
@@ -137,6 +218,26 @@ namespace eval reclaim {
         }
     }
 
+    # return the variations that would be used when upgrading a port
+    # installed with the given requested variants
+    proc get_variations {installed_variants} {
+        array set vararray {}
+        foreach v [array names macports::global_variations] {
+            set vararray($v) $macports::global_variations($v)
+        }
+        set splitvariant [split $installed_variants -]
+        set minusvariant [lrange $splitvariant 1 end]
+        set splitvariant [split [lindex $splitvariant 0] +]
+        set plusvariant [lrange $splitvariant 1 end]
+        foreach v $plusvariant {
+            set vararray($v) +
+        }
+        foreach v $minusvariant {
+            set vararray($v) -
+        }
+        return [array get varray]
+    }
+
     proc remove_distfiles {} {
         # Check for distfiles in both the root, and home directories. If found, delete them.
         # Args:
@@ -171,16 +272,27 @@ namespace eval reclaim {
         $progress start
 
         foreach port $installed_ports {
+            # skip additional versions installed with the same variants
+            if {[info exists seen([$port name],[$port requested_variants])]} {
+                continue
+            }
+            set seen([$port name],[$port requested_variants]) 1
+            array unset portinfo
             # Get mport reference
             try -pass_signal {
-                set mport [mportopen_installed [$port name] [$port version] [$port revision] [$port variants] {}]
+                if {[catch {mportlookup [$port name]} lookup_result] || [llength $lookup_result] < 2} {
+                     ui_warn [msgcat::mc "Port %s not found: %s" [$port name] $lookup_result]
+                    continue
+                }
+                array set portinfo [lindex $lookup_result 1]
+                set mport [mportopen $portinfo(porturl) [list subport $portinfo(name)] [get_variations [$port requested_variants]]]
             } catch {{*} eCode eMessage} {
                 $progress intermission
-                ui_warn [msgcat::mc "Failed to open port %s from registry: %s" [$port name] $eMessage]
+                ui_warn [msgcat::mc "Failed to open port %s %s: %s" [$port name] [$port requested_variants] $eMessage]
                 continue
             }
 
-            # Setup sub-Tcl-interpreter that executed the installed port
+            # Get sub-Tcl-interpreter that executed the installed port
             set workername [ditem_key $mport workername]
 
             # Append that port's distfiles to the list
@@ -192,7 +304,7 @@ namespace eval reclaim {
 
             foreach file [concat $distfiles $patchfiles] {
                 # split distfile into filename and disttag
-                set distfile [$workername eval "getdistname $file"]
+                set distfile [$workername eval [list getdistname $file]]
                 set root_path [file join $root_dist $dist_subdir $distfile]
                 set home_path [file join $home_dist $dist_subdir $distfile]
 
@@ -212,6 +324,8 @@ namespace eval reclaim {
             $progress update $i $port_count
             incr i
         }
+        array unset seen
+        array unset portinfo
 
         $progress finish
 
