@@ -17,7 +17,7 @@
  * 3. Neither the name of The MacPorts Project nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -100,7 +100,6 @@ typedef enum _copy_needed_return_t {
  *    SUID or SGID.
  *
  * @param path The absolute path of the binary to be executed
- * @param argc The number of arguments passed in \a argv
  * @param argv The arguments to be passed to the file to be executed
  * @param outargv Pointer to a modified array of arguments. Only valid if \c
  *                copy_is_needed is returned. May be \c NULL, in which case no
@@ -170,6 +169,7 @@ static copy_needed_return_t copy_needed(const char *path, char *const argv[],
             char *ctxt;
             char *word;
             size_t idx;
+            fclose(f);
             // do word splitting on the interpreter line and store it in new_argv
             for (idx = 0, word = strtok_r(linep, " \t\n", &ctxt);
                     word != NULL;
@@ -213,7 +213,11 @@ static copy_needed_return_t copy_needed(const char *path, char *const argv[],
                 // interpreter found, check that instead of given path
                 realpath = *new_argv;
             }
+        } else {
+            fclose(f);
         }
+    } else {
+        fclose(f);
     }
 
     // check whether the binary has SF_RESTRICTED and isn't SUID/SGID
@@ -343,10 +347,32 @@ static char *lazy_copy(const char *path, struct stat *in_st) {
     // we can safely copy the source file with all metadata.
     // This cannot use COPYFILE_CLONE as it does not follow symlinks,
     // see https://trac.macports.org/ticket/55575
-    if (copyfile(path, target_path_temp, NULL, COPYFILE_ALL) != 0) {
+    if (copyfile(path, target_path_temp, NULL, COPYFILE_ACL | COPYFILE_XATTR | COPYFILE_DATA | COPYFILE_EXCL) != 0) {
         fprintf(stderr, "sip_copy_proc: copyfile(%s, %s): %s\n", path, target_path_temp, strerror(errno));
         goto lazy_copy_out;
     }
+
+    // Re-open the copied file in outfd, because copyfile(3) with COPYFILE_ALL
+    // (or COPYFILE_STAT, for that matter) does not seem to copy the file
+    // creation time correctly, so we need another futimes(2) to fix that.
+#ifdef O_CLOEXEC
+    if (-1 == (outfd = open(target_path_temp, O_RDWR | O_CLOEXEC))) {
+        fprintf(stderr, "sip_copy_proc: open(%s, O_RDWR | O_CLOEXEC): %s\n", target_path_temp, strerror(errno));
+#else
+    if (-1 == (outfd = open(target_path_temp, O_RDWR))) {
+        fprintf(stderr, "sip_copy_proc: open(%s, O_RDWR): %s\n", target_path_temp, strerror(errno));
+#endif
+        goto lazy_copy_out;
+    }
+
+    // Since we removed COPYFILE_STAT from the copyfile(3) invocation above, we
+    // need to restore the permissions on the copy. Note that the futimes(2)
+    // later on should still succeed even if we remove write permissions here,
+    // because we already have a file descriptor open.
+    if (-1 == fchmod(outfd, in_st->st_mode)) {
+        fprintf(stderr, "sip_copy_proc: fchmod(%s, %o): %s\n", target_path_temp, in_st->st_mode, strerror(errno));
+        goto lazy_copy_out;
+    }
 #else /* !HAVE_COPYFILE */
     // create temporary file to copy into and then later atomically replace
     // target file
@@ -407,6 +433,7 @@ static char *lazy_copy(const char *path, struct stat *in_st) {
     if (bytes_read < 0 || bytes_written < 0) {
         goto lazy_copy_out;
     }
+#endif /* HAVE_COPYFILE */
 
     struct timeval times[2];
     TIMESPEC_TO_TIMEVAL(&times[0], &in_st->st_mtimespec);
@@ -415,7 +442,6 @@ static char *lazy_copy(const char *path, struct stat *in_st) {
         fprintf(stderr, "sip_copy_proc: futimes(%s): %s\n", target_path_temp, strerror(errno));
         goto lazy_copy_out;
     }
-#endif /* HAVE_COPYFILE */
 
     if (-1 == rename(target_path_temp, target_path)) {
         fprintf(stderr, "sip_copy_proc: rename(%s, %s): %s\n", target_path_temp, target_path, strerror(errno));
@@ -474,6 +500,7 @@ int sip_copy_execve(const char *path, char *const argv[], char *const envp[]) {
 
                 char *new_path = lazy_copy(to_be_copied, &st);
                 if (!new_path) {
+                    free_argv(outargv);
                     return -1;
                 }
 
@@ -523,6 +550,7 @@ int sip_copy_posix_spawn(
 
                 char *new_path = lazy_copy(to_be_copied, &st);
                 if (!new_path) {
+                    free_argv(outargv);
                     return -1;
                 }
 
