@@ -88,7 +88,7 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
     if {[info exists options(ports_activate_no-exec)]} {
         set noexec $options(ports_activate_no-exec)
     }
-    set rename_list {}
+    set rename_list [list]
     if {[info exists options(portactivate_rename_files)]} {
         set rename_list $options(portactivate_rename_files)
     }
@@ -109,6 +109,11 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
         set specifier "${version}_${revision}${variants}"
         set location [$requested location]
 
+        if {[$requested state] eq "installed"} {
+            ui_info "${name} @${specifier} is already active."
+            return
+        }
+
         # if another version of this port is active, deactivate it first
         set current [registry::entry installed $name]
         foreach i $current {
@@ -124,9 +129,6 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
         if {![::file isfile $location]} {
             return -code error "Image error: Can't find image file $location"
         }
-        if {[$requested state] eq "installed"} {
-            return -code error "Image error: ${name} @${specifier} is already active."
-        }
     }
     foreach a $todeactivate {
         if {$noexec || ![registry::run_target $a deactivate [list ports_nodepcheck 1]]} {
@@ -173,7 +175,20 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
     if { [llength $ilist] == 1 } {
         set requested [lindex $ilist 0]
     } else {
-        throw registry::image-error "Image error: port ${name} is not active."
+        set ilist [_check_registry $name $version $revision $variants 1]
+        if {[llength $ilist] > 0} {
+            ui_info "port ${name} is already inactive"
+            return
+        } else {
+            set v ""
+            if {$version ne ""} {
+                set v " @${version}"
+                if {$revision ne ""} {
+                    append v _${revision}${variants}
+                }
+            }
+            throw registry::image-error "Image error: port ${name}${v} is not active."
+        }
     }
     # set name again since the one we were passed may not have had the correct case
     set name [$requested name]
@@ -185,6 +200,18 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
         if {$revision ne ""} {
             append v _${revision}${variants}
         }
+        set ilist [_check_registry $name $version $revision $variants 1]
+        foreach inact $ilist {
+            if {$revision ne ""} {
+                set thisv [$inact version]_[$inact revision][$inact variants]
+            } else {
+                set thisv [$inact version]
+            }
+            if {$v eq $thisv} {
+                ui_info "port ${name} @${thisv} is already inactive"
+                return
+            }
+        }
         return -code error "Active version of $name is not $v but ${specifier}."
     }
 
@@ -208,7 +235,7 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
     _deactivate_contents $requested [$requested files] $force
 }
 
-proc _check_registry {name version revision variants} {
+proc _check_registry {name version revision variants {return_all 0}} {
     global UI_PREFIX
 
     set searchkeys $name
@@ -224,9 +251,12 @@ proc _check_registry {name version revision variants} {
         }
     }
     set ilist [registry::entry imaged {*}$searchkeys]
+    if {$return_all} {
+        return $ilist
+    }
 
     if { [llength $ilist] > 1 } {
-        set portilist {}
+        set portilist [list]
         set msg "The following versions of $name are currently installed:"
         if {[macports::ui_isset ports_noninteractive]} {
             ui_msg "$UI_PREFIX [msgcat::mc $msg]"
@@ -336,13 +366,13 @@ proc extract_archive_to_tmpdir {location} {
                         set gzip "gzip"
                         if {[catch {set gzip [macports::findBinary $gzip ${macports::autoconf::gzip_path}]} errmsg] == 0} {
                             ui_debug "Using $gzip"
-                            set unarchive.pipe_cmd "$gzip -d -c ${location} |"
+                            set unarchive.pipe_cmd "$gzip -d -c [macports::shellescape ${location}] |"
                         } else {
                             ui_debug $errmsg
                             throw MACPORTS "No '$gzip' was found on this system!"
                         }
                     } else {
-                        set unarchive.args "-f ${location} ."
+                        set unarchive.args "-f [macports::shellescape ${location}] ."
                     }
                 } else {
                     ui_debug $errmsg
@@ -353,7 +383,10 @@ proc extract_archive_to_tmpdir {location} {
                 global macports::hfscompression
                 # Opportunistic HFS compression. bsdtar will automatically
                 # disable this if filesystem does not support compression.
-                if {${macports::hfscompression} &&
+                # Don't use if not running as root, due to bugs:
+                # The system bsdtar on 10.15 suffers from https://github.com/libarchive/libarchive/issues/497
+                # Later versions fixed that problem but another remains: https://github.com/libarchive/libarchive/issues/1415 
+                if {${macports::hfscompression} && [getuid] == 0 &&
                         ![catch {macports::binaryInPath bsdtar}] &&
                         ![catch {exec bsdtar -x --hfsCompression < /dev/null >& /dev/null}]} {
                     ui_debug "Using bsdtar with HFS+ compression (if valid)"
@@ -394,13 +427,13 @@ proc extract_archive_to_tmpdir {location} {
                     }
                     if {[catch {set gzip [macports::findBinary $gzip $hint]} errmsg] == 0} {
                         ui_debug "Using $gzip"
-                        set unarchive.pipe_cmd "$gzip -d -c ${location} |"
+                        set unarchive.pipe_cmd "$gzip -d -c [macports::shellescape ${location}] |"
                     } else {
                         ui_debug $errmsg
                         throw MACPORTS "No '$gzip' was found on this system!"
                     }
                 } else {
-                    set unarchive.args "${location}"
+                    set unarchive.args [macports::shellescape ${location}]
                 }
             }
             xar {
@@ -409,7 +442,7 @@ proc extract_archive_to_tmpdir {location} {
                     ui_debug "Using $xar"
                     set unarchive.cmd "$xar"
                     set unarchive.pre_args {-xvpf}
-                    set unarchive.args "${location}"
+                    set unarchive.args [macports::shellescape ${location}]
                 } else {
                     ui_debug $errmsg
                     throw MACPORTS "No '$xar' was found on this system!"
@@ -425,7 +458,7 @@ proc extract_archive_to_tmpdir {location} {
                     } else {
                         set unarchive.pre_args {-o}
                     }
-                    set unarchive.args "${location} -d ."
+                    set unarchive.args "[macports::shellescape ${location}] -d ."
                 } else {
                     ui_debug $errmsg
                     throw MACPORTS "No '$unzip' was found on this system!"
@@ -560,7 +593,7 @@ proc _activate_contents {port {rename_list {}}} {
         # debug output of activate make more sense.
         set files [lsort -increasing -unique $files]
         # handle files that are to be renamed
-        set confirmed_rename_list {}
+        set confirmed_rename_list [list]
         foreach {src dest} $rename_list {
             set index [lsearch -exact -sorted $files $src]
             if {$index != -1} {
@@ -568,7 +601,7 @@ proc _activate_contents {port {rename_list {}}} {
                 lappend confirmed_rename_list $src $dest
             }
         }
-        set rollback_filelist {}
+        set rollback_filelist [list]
 
         registry::write {
             # Activate it, and catch errors so we can roll-back
@@ -690,7 +723,7 @@ proc _deactivate_contents {port imagefiles {force 0} {rollback 0}} {
     set files [list]
 
     foreach file $imagefiles {
-        if { [::file exists $file] || (![catch {::file type $file}] && [::file type $file] eq "link") } {
+        if { [::file exists $file] || (![catch {::file type $file} ft] && $ft eq "link") } {
             # Normalize the file path to avoid removing the intermediate
             # symlinks (remove the empty directories instead)
             # Remark: paths in the registry may be not normalized.
